# Generated by Django 2.2 on 2019-07-09 16:43

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('anss', '0003_auto_20190709_0939'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='feed',
            options={'get_latest_by': 'archived_datetime', 'ordering': ('-archived_datetime',), 'verbose_name': 'Archived feed'},
        ),
        migrations.AlterModelOptions(
            name='feedearthquake',
            options={'get_latest_by': ('-feed_id', '-time'), 'ordering': ('-feed_id', '-time'), 'verbose_name': 'Archived earthquake'},
        ),
    ]



# Generated by Django 2.2 on 2019-07-09 17:35

import django.contrib.gis.db.models.fields
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('anss', '0004_auto_20190709_0943'),
    ]

    operations = [
        migrations.AlterField(
            model_name='feedearthquake',
            name='alert',
            field=models.CharField(blank=True, help_text='The alert level from the PAGER earthquake impact scale', max_length=5000, verbose_name='alert level'),
        ),
        migrations.AlterField(
            model_name='feedearthquake',
            name='cdi',
            field=models.FloatField(help_text='The maximum reported intensity for the event. Computed by DYFI. While typically reported as a roman numeral, for the purposes of this API, intensity is expected as the decimal equivalent of the roman numeral', null=True, verbose_name='community decimal intensity'),
        ),
        migrations.AlterField(
            model_name='feedearthquake',
            name='code',
            field=models.CharField(blank=True, help_text='An identifying code assigned by - and unique from - the corresponding source for the event.', max_length=5000, verbose_name='unique ID'),
        ),
        migrations.AlterField(
            model_name='feedearthquake',
            name='depth',
            field=models.FloatField(help_text='Depth of the event in kilometers', null=True),
        ),
        migrations.AlterField(
            model_name='feedearthquake',
            name='dmin',
            field=models.FloatField(help_text='Horizontal distance from the epicenter to the nearest station (in degrees). 1 degree is approximately 111.2 kilometers. In general, the smaller this number, the more reliable is the calculated depth of the earthquake.', null=True, verbose_name='nearest seismic station'),
        ),
        migrations.AlterField(
            model_name='feedearthquake',
            name='feed',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='anss.Feed', verbose_name='archived source'),
        ),
        migrations.AlterField(
            model_name='feedearthquake',
            name='felt',
            field=models.IntegerField(help_text='The total number of felt reports submitted to the DYFI? system', null=True, verbose_name='how many felt it?'),
        ),
        migrations.AlterField(
            model_name='feedearthquake',
            name='gap',
            field=models.FloatField(help_text='The largest azimuthal gap between azimuthally adjacent stations (in degrees). In general, the smaller this number, the more reliable is the calculated horizontal position of the earthquake. Earthquake locations in which the azimuthal gap exceeds 180 degrees typically have large location and depth uncertainties.', null=True, verbose_name='largest azimuthal gap'),
        ),
        migrations.AlterField(
            model_name='feedearthquake',
            name='ids',
            field=models.CharField(blank=True, help_text='A comma-separated list of event ids that are associated to an event.', max_length=5000, verbose_name='identifiers'),
        ),
        migrations.AlterField(
            model_name='feedearthquake',
            name='mag',
            field=models.FloatField(null=True, verbose_name='magnitude'),
        ),
        migrations.AlterField(
            model_name='feedearthquake',
            name='magType',
            field=models.CharField(blank=True, help_text='The method or algorithm used to calculate the preferred magnitude for the event.', max_length=5000, verbose_name='magnitude type'),
        ),
        migrations.AlterField(
            model_name='feedearthquake',
            name='mmi',
            field=models.FloatField(help_text='The maximum estimated instrumental intensity for the event. Computed by ShakeMap. While typically reported as a roman numeral, for the purposes of this API, intensity is expected as the decimal equivalent of the roman numeral.', null=True, verbose_name='Maximum Modified Mercalli Intensity'),
        ),
        migrations.AlterField(
            model_name='feedearthquake',
            name='net',
            field=models.CharField(blank=True, help_text='The ID of a data contributor. Identifies the network considered to be the preferred source of information for this event.', max_length=5000, verbose_name='network ID'),
        ),
        migrations.AlterField(
            model_name='feedearthquake',
            name='nst',
            field=models.IntegerField(help_text='The total number of seismic stations used to determine earthquake location', null=True, verbose_name='seismic stations'),
        ),
        migrations.AlterField(
            model_name='feedearthquake',
            name='place',
            field=models.CharField(blank=True, help_text='Description of the epicenter', max_length=5000),
        ),
        migrations.AlterField(
            model_name='feedearthquake',
            name='point',
            field=django.contrib.gis.db.models.fields.PointField(null=True, srid=4326, verbose_name='epicenter'),
        ),
        migrations.AlterField(
            model_name='feedearthquake',
            name='rms',
            field=models.FloatField(help_text='The root-mean-square (RMS) travel time residual, in sec, using all weights. This parameter provides a measure of the fit of the observed arrival times to the predicted arrival times for this location. Smaller numbers reflect a better fit of the data. The value is dependent on the accuracy of the velocity model used to compute the earthquake location, the quality weights assigned to the arrival time data, and the procedure used to locate the earthquake.', null=True, verbose_name='root mean squared'),
        ),
        migrations.AlterField(
            model_name='feedearthquake',
            name='sig',
            field=models.IntegerField(help_text='A number describing how significant the event is. Larger numbers indicate a more significant event. This value is determined on a number of factors, including: magnitude, maximum MMI, felt reports, and estimated impact.', null=True, verbose_name='significance'),
        ),
        migrations.AlterField(
            model_name='feedearthquake',
            name='sources',
            field=models.CharField(blank=True, help_text='A comma-separated list of network contributors', max_length=5000, verbose_name='network sources'),
        ),
        migrations.AlterField(
            model_name='feedearthquake',
            name='status',
            field=models.CharField(blank=True, help_text='Indicates whether the event has been reviewed by a human. Status is either automatic or reviewed. Automatic events are directly posted by automatic processing systems and have not been verified or altered by a human. Reviewed events have been looked at by a human. The level of review can range from a quick validity check to a careful reanalysis of the eventself.', max_length=5000),
        ),
        migrations.AlterField(
            model_name='feedearthquake',
            name='time',
            field=models.BigIntegerField(help_text='Time when the event occurred. Times are reported in milliseconds since the epoch.', null=True, verbose_name='occurred at (UNIX)'),
        ),
        migrations.AlterField(
            model_name='feedearthquake',
            name='title',
            field=models.CharField(blank=True, help_text='a summary of the event', max_length=5000, verbose_name='description'),
        ),
        migrations.AlterField(
            model_name='feedearthquake',
            name='tsunami',
            field=models.IntegerField(help_text='This flag is set to 1 for large events in oceanic regions and 0 otherwise. The existence or value of this flag does not indicate if a tsunami actually did or will exist.', null=True, verbose_name='tsunami warning'),
        ),
        migrations.AlterField(
            model_name='feedearthquake',
            name='type',
            field=models.CharField(blank=True, help_text='Type of seismic event', max_length=5000, verbose_name='event type'),
        ),
        migrations.AlterField(
            model_name='feedearthquake',
            name='types',
            field=models.CharField(blank=True, help_text='A comma-separated list of product types associated to this event', max_length=5000, verbose_name='product types'),
        ),
        migrations.AlterField(
            model_name='feedearthquake',
            name='tz',
            field=models.IntegerField(help_text='Timezone offset from UTC in minutes at the event epicenter', null=True),
        ),
        migrations.AlterField(
            model_name='feedearthquake',
            name='updated',
            field=models.BigIntegerField(help_text='Time when the event was most recently updated. Times are reported in milliseconds since the epoch.', null=True, verbose_name='updated at (UNIX)'),
        ),
        migrations.AlterField(
            model_name='feedearthquake',
            name='usgs_id',
            field=models.CharField(blank=True, help_text='A composite identifier that combines the source network and the earthquake.', max_length=5000, verbose_name='USGS ID'),
        ),
    ]



# Generated by Django 2.2 on 2019-07-09 15:30

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('anss', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='feedearthquake',
            options={'get_latest_by': ('-feed_id', '-time'), 'ordering': ('-feed_id', '-time')},
        ),
    ]



# Generated by Django 2.2 on 2019-07-09 05:45

import django.contrib.gis.db.models.fields
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Feed',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('archived_datetime', models.DateTimeField(help_text='The time the feed was pulled.', null=True)),
                ('type', models.CharField(choices=[('m1', 'Magnitude > 1.0')], max_length=500)),
                ('format', models.CharField(choices=[('geojson', 'GeoJSON')], max_length=500)),
                ('timeframe', models.CharField(choices=[('one-hour', 'One hour')], max_length=500)),
                ('content', models.FileField(upload_to='')),
                ('generated', models.BigIntegerField(null=True)),
                ('url', models.CharField(blank=True, max_length=5000)),
                ('title', models.CharField(blank=True, max_length=5000)),
                ('api', models.CharField(blank=True, max_length=5000)),
                ('count', models.IntegerField(null=True)),
                ('status', models.IntegerField(null=True)),
            ],
            options={
                'ordering': ('-archived_datetime',),
                'get_latest_by': 'archived_datetime',
            },
        ),
        migrations.CreateModel(
            name='FeedEarthquake',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('mag', models.FloatField(null=True)),
                ('place', models.CharField(blank=True, max_length=5000)),
                ('time', models.BigIntegerField(null=True)),
                ('updated', models.BigIntegerField(null=True)),
                ('tz', models.IntegerField(null=True)),
                ('url', models.CharField(blank=True, max_length=5000)),
                ('detail', models.CharField(blank=True, max_length=5000)),
                ('felt', models.IntegerField(null=True)),
                ('cdi', models.FloatField(null=True)),
                ('mmi', models.FloatField(null=True)),
                ('alert', models.CharField(blank=True, max_length=5000)),
                ('status', models.CharField(blank=True, max_length=5000)),
                ('tsunami', models.IntegerField(null=True)),
                ('sig', models.IntegerField(null=True)),
                ('net', models.CharField(blank=True, max_length=5000)),
                ('code', models.CharField(blank=True, max_length=5000)),
                ('ids', models.CharField(blank=True, max_length=5000)),
                ('sources', models.CharField(blank=True, max_length=5000)),
                ('types', models.CharField(blank=True, max_length=5000)),
                ('nst', models.IntegerField(null=True)),
                ('dmin', models.FloatField(null=True)),
                ('rms', models.FloatField(null=True)),
                ('gap', models.FloatField(null=True)),
                ('magType', models.CharField(blank=True, max_length=5000)),
                ('type', models.CharField(blank=True, max_length=5000)),
                ('title', models.CharField(blank=True, max_length=5000)),
                ('depth', models.FloatField(null=True)),
                ('point', django.contrib.gis.db.models.fields.PointField(null=True, srid=4326)),
                ('usgs_id', models.CharField(blank=True, max_length=5000)),
                ('feed', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='anss.Feed')),
            ],
            options={
                'ordering': ('-feed_id', '-time'),
            },
        ),
    ]



# Django helpers
from django.utils import timezone
from django.contrib.gis.geos import Point
from anss.models import Feed, FeedEarthquake
from django.core.management.base import BaseCommand, CommandError

# Files
import requests
from django.core.files.base import ContentFile

# Logging
import logging
logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = (
        "Archive the latest real-time earthquake notifications "
        "from the USGS's Advanced National Seismic System"
    )

    def set_options(self, *args, **options):
        self.now = timezone.now()
        self.url = "https://earthquake.usgs.gov/earthquakes/feed/v1.0/summary/1.0_hour.geojson"
        self.feed = Feed.objects.create(
            archived_datetime=self.now,
            type="m1",
            format="geojson",
            timeframe="one-hour",
        )
        self.feedearthquake_model = self.get_feedearthquake_model()

    def handle(self, *args, **options):
        # Set options
        self.set_options(*args, **options)

        # Get the feed
        raw_feed = self.get_usgs_feed()

        # Check the response
        logger.debug(f"Response code: {raw_feed.status_code}")
        if not raw_feed.status_code == 200:
            msg = f"Request for {self.url} failed with code {raw_feed.status_code}"
            logger.error(msg)
            self.feed.status = raw_feed.status_code
            self.feed.save()
            raise CommandError(msg)

        # Save the file
        logger.debug("Archiving data")
        self.feed.content.save(
            self.get_file_path(),
            ContentFile(raw_feed.content),
            save=True
        )
        logger.debug(f"Archived at {self.feed.content.url}")

        # Read in the GeoJSON as a Python dictionary
        geojson = raw_feed.json()

        # Save the metadata to the database
        metadata = geojson['metadata']
        logger.debug(f"Logging metadata {metadata}")
        self.feed.generated = metadata['generated']
        self.feed.url = metadata['url']
        self.feed.title = metadata['title']
        self.feed.api = metadata['api']
        self.feed.count = metadata['count']
        self.feed.status = metadata['status']
        self.feed.save()

        # Save each earthquake
        [self.create_feedearthquake(d) for d in geojson['features']]

    def safestr(self, v):
        """
        Safely prepare a string value from the source data for the database.
        """
        return (v or "").strip()

    def get_feedearthquake_model(self):
        """
        Returns the model that will be used to save the FeedEarthquake data.
        """
        return FeedEarthquake

    def get_usgs_feed(self):
        """
        Returns a real-time feed from ANSS.
        """
        # Request the URL
        logger.debug(f"Requesting {self.url}")
        return requests.get(self.url)

    def get_file_path(self):
        """
        Returns the file path where the archived content will be saved in the MEDIA_ROOT
        """
        return f"anss/{self.feed.type}/{self.feed.format}/{self.feed.timeframe}/{self.feed.archived_datetime}.json"

    def create_feedearthquake(self, d):
        """
        Accepts a raw GeoJSON feature dictionary from the an ANSS real-time feed and creates a database record.
        """
        p = d['properties']
        obj = self.feedearthquake_model(
            feed=self.feed,
            mag=p['mag'],
            place=self.safestr(p['place']),
            time=p['time'],
            updated=p['updated'],
            tz=p['tz'],
            url=self.safestr(p['url']),
            detail=self.safestr(p['detail']),
            felt=p['felt'],
            cdi=p['cdi'],
            mmi=p['mmi'],
            alert=self.safestr(p['alert']),
            status=p['status'],
            tsunami=p['tsunami'],
            sig=p['sig'],
            net=self.safestr(p['net']),
            code=self.safestr(p['code']),
            ids=self.safestr(p['ids']),
            sources=self.safestr(p['sources']),
            types=self.safestr(p['types']),
            nst=p['nst'],
            dmin=p['dmin'],
            rms=p['rms'],
            gap=p['gap'],
            magType=self.safestr(p['magType']),
            type=self.safestr(p['type']),
            title=self.safestr(p['title']),
            usgs_id=self.safestr(d['id'])
        )
        lng, lat, depth = d['geometry']['coordinates']
        obj.point = Point(lng, lat)
        obj.depth = depth
        obj.save()
        logger.debug(f"Saved {obj}")
        return obj
